/*! \file
    \brief Command and Error codes for BDM communication over USB
*/
#ifndef _COMMANDS_H_
#define _COMMANDS_H_

/* command format:

1   byte : command number (see below)
n   bytes: command parameters (data)

   data format:

all 16-bit and 32-bit data is transferred in big endian, i.e. MSB on lower address (first) and LSB on higher address (next)
data must be converted properly on intel machines (PCs)

*/

/* cable status bit fields */
#define RESET_DETECTED_MASK   0x0001
#define RSTO_STATE_MASK       0x0002

/* target types */
#define TARGET_TYPE_CF_BDM    0
#define TARGET_TYPE_JTAG      1


//! \brief Maximum USB transfer size - entire transfer!
//!
//! Limited by RAM, Multiple of 64 is nice since transfers may be in 8/64-byte pieces.
#define MAX_PACKET_SIZE       (2*64)
#define MAX_DATA_SIZE         (MAX_PACKET_SIZE-5) //!< Max data transfer (HC12/HC08/RS08)
#define MAX_CF_DATA_SIZE      (MAX_PACKET_SIZE-7) //!< Max data transfer (Coldfire V1)
#define MAX_CFx_DATA_SIZE      (MAX_PACKET_SIZE-1) //!< Max data transfer (Coldfire V2,3,4)

/*
 * If command fails, the device responds with command code CMD_FAILED
 * If command succeeds, the device responds with the same command number followed by any results
 *     as appropriate
*/
//! BDM command values
//!
//! The following values are the 1st byte in each command.  Other parameters are as shown below. \n
//!  Each command returns a status value (see \ref  ErrorCodes) as the first byte followed by any results as indicated below.
//!
enum BDMCommands{
   // The following few were actually used as error codes - replaced by BDM_RC_...
   //CMD_OK         = 0,       //!< deprecated
   //CMD_ERR_SUM    = 1,       //!< deprecated, USB has CRC16 in place
   CMD_FAILED     = 2,         //!< deprecated, command execution failed (incorrect parameters,  target not responding, etc.)
   CMD_UNKNOWN    = 3,         //!< deprecated, unknown command

   CMD_DEBUG      = 8,        //!< Debugging commands (parameter determines actual command) @param [2]  Debug command see \ref DebugSubCommands
   CMD_OPTION     = 10,       //!< Set/clear various options (only has affect after next \ref CMD_SET_TARGET) @param [2]  Option command see \ref OptionCommands
   CMD_GET_VER          = 12, //!< Get firmware version in BCD @return [1] 8-bit HW (major+minor) revision \n [2] 8-bit SW (major+minor) version number
   CMD_GET_LAST_STATUS  = 13, //!< Get status from last command @return [0] 8-bit Error code see \ref  ErrorCodes

   CMD_SET_BOOT         = 14, //!< Request bootloader firmware upgrade on future power-ups. This is a permanent change. @param [2..5] must be "BOOT"

/* BDM/debugging related commands */
CMD_SET_TARGET        = 20, /* set target, 8bit parameter: 00=ColdFire(default), 01=JTAG */
CMD_RESET             = 21, /* 8bit parameter: 0=reset to BDM Mode, 1=reset to Normal mode */
CMD_GET_STATUS        = 22, /* returns 16bit status word: bit0 - target was reset since last execution of this command (this bit is cleared after reading), bit1 - current state of the RSTO pin, big endian! */
CMD_HALT              = 23, /* stop the CPU and bring it into BDM mode */
CMD_GO                = 24, /* start code execution from current PC address */
CMD_STEP              = 25, /* perform single step */
CMD_RESYNCHRONIZE     = 26, /* resynchronize communication with the target (in case of noise, etc.) */
CMD_ASSERT_TA         = 27, /* parameter: 8-bit number of 10us ticks - duration of the TA assertion */

/* CPU related commands */
CMD_READ_MEM8         = 30, /* parameter 32bit address, returns 8bit value read from address */
CMD_READ_MEM16        = 31, /* parameter 32bit address, returns 16bit value read from address */
CMD_READ_MEM32        = 32, /* parameter 32bit address, returns 32bit value read from address */

CMD_READ_MEMBLOCK8    = 33, /* parameter 32bit address, returns block of 8bit values read from the address and onwards */
CMD_READ_MEMBLOCK16   = 34, /* parameter 32bit address, returns block of 16bit values read from the address and onwards */
CMD_READ_MEMBLOCK32   = 35, /* parameter 32bit address, returns block of 32bit values read from the address and onwards */

CMD_WRITE_MEM8        = 36, /* parameter 32bit address & an 8-bit value to be written to the address */
CMD_WRITE_MEM16       = 37, /* parameter 32bit address & a 16-bit value to be written to the address */
CMD_WRITE_MEM32       = 38, /* parameter 32bit address & a 32-bit value to be written to the address */

CMD_WRITE_MEMBLOCK8   = 39, /* parameter 32bit address & a block of 8-bit values to be written from the address */
CMD_WRITE_MEMBLOCK16  = 40, /* parameter 32bit address & a block of 16-bit values to be written from the address */
CMD_WRITE_MEMBLOCK32  = 41, /* parameter 32bit address & a block of 32-bit values to be written from the address */

CMD_READ_REG          = 42, /* parameter 8-bit register number to read, returns 32-bit register contents */
CMD_WRITE_REG         = 43, /* parameter 8-bit register number to write & the 32-bit register contents to be written */
CMD_READ_CREG         = 44, /* parameter 16-bit register address, returns 32-bit control register contents */
CMD_WRITE_CREG        = 45, /* parameter 16-bit register address & the 32-bit control register contents to be written */

CMD_READ_DREG         = 46, /* parameter 8-bit register number to read, returns 32-bit debug module register contents */
CMD_WRITE_DREG        = 47, /* parameter 8-bit register number to write & the 32-bit debug module register contents to be written */

/* JTAG commands */
CMD_JTAG_GOTORESET    = 80, /* no parameters, takes the TAP to TEST-LOGIC-RESET state, re-select the JTAG target to take TAP back to RUN-TEST/IDLE */
CMD_JTAG_GOTOSHIFT    = 81, /* parameters 8-bit path option; path option ==0 : go to SHIFT-DR, !=0 : go to SHIFT-IR (requires the tap to be in RUN-TEST/IDLE) */
CMD_JTAG_WRITE        = 82, /* parameters 8-bit exit option, 8-bit count of bits to shift in, and the data to be shifted in (shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte; exit option ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when finished */
CMD_JTAG_READ         = 83, /* parameters 8-bit exit option, 8-bit count of bits to shift out; exit option ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when finished, returns the data read out of the device (first bit in LSB of the last byte in the buffer) */
};


//! Error codes returned from BDM routines and BDM commands.
enum ErrorCodes {
  BDM_RC_OK                      = 0,     //!< - No error
  BDM_RC_ILLEGAL_PARAMS          = 1,     //!< - Illegal parameters to command
  BDM_RC_FAIL                    = 2,     //!< - General Fail - only used by CMD_GET_LAST_STATUS - don't change
  BDM_RC_BUSY                    = 3,     //!< - Busy with last command - try again - don't change
  BDM_RC_ILLEGAL_COMMAND         = 4,     //!< - Illegal (unknown) command (may be in wrong target mode)
  BDM_RC_NO_CONNECTION           = 5,     //!< - No connection to target
  BDM_RC_OVERRUN                 = 6,     //!< - New command before previous command completed
  // Gap to avoid clash with used command codes CMD_...
  BDM_RC_UNKNOWN_TARGET          = 15,    //!< - Target not supported
  BDM_RC_NO_TX_ROUTINE           = 16,    //!< - No Tx routine available at measured BDM communication speed
  BDM_RC_NO_RX_ROUTINE           = 17,    //!< - No Rx routine available at measured BDM communication speed
  BDM_RC_BDM_EN_FAILED           = 18,    //!< - Failed to enable BDM mode in target
  BDM_RC_RESET_TIMEOUT           = 19,    //!< - RESET signal failed to rise/fall
  BDM_RC_BKGD_TIMEOUT            = 20,    //!< - BKGD signal failed to rise/fall
  BDM_RC_SYNC_TIMEOUT            = 21,    //!< - No response to SYNC sequence
  BDM_RC_UNKNOWN_SPEED           = 22,    //!< - Communication speed is not known or cannot be determined
  BDM_RC_WRONG_PROGRAMMING_MODE  = 23,    //!< - Attempted Flash programming when in wrong mode (e.g. Vpp off)
  BDM_RC_FLASH_PROGRAMING_BUSY   = 24,    //!< - Busy with last Flash programming command
  BDM_RC_VDD_NOT_REMOVED         = 25,    //!< - Target Vdd failed to fall
  BDM_RC_VDD_NOT_PRESENT         = 26,    //!< - Target Vdd not present/failed to rise
  BDM_RC_VDD_WRONG_MODE          = 27,    //!< - Attempt to cycle target Vdd when not controlled by BDM interface
};

//! Option sub-commands (used with \ref CMD_OPTION)
enum OptionCommands {
   BDM_GET_CAPABILITIES                = 0, //!< - Returns capability vector for hardware (see \ref HardwareCapabilities)
   BDM_OPT_SET_TARGET_VDD              = 1, //!< - Set target Vdd (see \ref TargetVddSelection)
   BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT = 2, //!< - Enable/disable Cycling Target Vdd if connection problems
   BDM_OPT_AUTOMATIC_RECONNECT         = 3, //!< - Enable/disable automatic reconnect
   BDM_OPT_GUESS_SPEED                 = 4, //!< - Enable/disable guessing speed for target w/o ACKN feature (HCS12)
   BDM_OPT_LEAVE_TARGET_POWERED        = 5, //!< - Enable/disable leaving target powered when exiting
   BDM_OPT_USE_ALT_BDM_CLOCK           = 6, //!< - Enable/disable using alternative BDM clock source in target (HCS08)
   BDM_OPT_CYCLE_TARGET_VDD_ON_RESET   = 7, //!< - Enable/disable Cycling Target Vdd  on Reset
   BDM_OPT_USE_RESET_SIGNAL            = 8, //!< - Enable/disable use of RESET signal on BDM interface
   BDM_OPT_SET_CLOCK_TRIM_FREQ         = 9, //!< - Set clock frequency to trim to
   BDM_OPT_SET_INTERFACELEVEL          = 10, //!< - Directly set the level on the BDM interface
};

//! Capabilities for the hardware (used with \ref BDM_GET_CAPABILITIES)
enum HardwareCapabilities {
  BDM_CAP_RESET        = (1<<0),   //!< - RESET can be driven/sensed
  BDM_CAP_FLASH        = (1<<1),   //!< - 12 V Flash programming supply available (RS08)
  BDM_CAP_VDDCONTROL   = (1<<2),   //!< - Control over target Vdd
  BDM_CAP_VDDSENSE     = (1<<3),   //!< - Sensing of target Vdd
} ;

//! Target Voltage selection (used with \ref BDM_OPT_SET_TARGET_VDD )
enum TargetVddSelection {
   BDM_TARGET_VDD_OFF      = 0,   //!< - Set target Vdd Off
   BDM_TARGET_VDD_3V3      = 1,   //!< - Set target Vdd to 3V3
   BDM_TARGET_VDD_5V       = 2,   //!< - Set target Vdd to 5V
};

//! Target BDM Clock selection (used with \ref BDM_OPT_USE_ALT_BDM_CLOCK )
enum ClkSwValues {
   DEFAULT   = -1,
   ALT       = 0,
   NORMAL    = 1,
} ;

//! Debugging sub commands (used with \ref CMD_DEBUG )
//! @note Not for general use! (Dangerous - don't try turning on VPP with the wrong chip!)
enum DebugSubCommands {
  BDM_DBG_ACKN             = 0,  //!< - Test ACKN
  BDM_DBG_SYNC             = 1,  //!< - Test SYNC
  BDM_DBG_TESTPORT         = 2,  //!< - Test BDM port timing
  BDM_DBG_USBDISCONNECT    = 3,  //!< - Test USB disconnect (don't use!)
  BDM_DBG_STACKSIZE        = 4,  //!< - Determine stack size
  BDM_DBG_VPP_OFF          = 5,  //!< - Remove Flash programming voltage from target
  BDM_DBG_VPP_ON           = 6,  //!< - Apply Flash programming voltage to target
  BDM_DBG_FLASH12V_OFF     = 7,  //!< - Turn 12V flash programming voltage source off
  BDM_DBG_FLASH12V_ON      = 8,  //!< - Turn 12V flash programming voltage source on
  BDM_DBG_VDD_OFF          = 9,  //!< - Turn Target Vdd supply off
  BDM_DBG_VDD3_ON          = 10, //!< - Set  Target Vdd supply to 3V3
  BDM_DBG_VDD5_ON          = 11, //!< - Set Target Vdd supply to 5V
  BDM_DBG_CYCLE_POWER      = 12, //!< - Cycle Target Vdd supply off and on
  BDM_DBG_MEASURE_VDD      = 13, //!< - Measure Target Vdd supply
  BDM_DBG_RS08TRIM         = 14, //!< - Calculate RS08 clock trim value
  BDM_DBG_TESTWAITS        = 15, //!< - Tests the software counting delays used for BDM communication. (locks up BDM!)
  BDM_DBG_TESTALTSPEED     = 16, //!< - Test bdmHC12_alt_speed_detect()
};

//! Error codes returned by JMxx BDM when in ICP mode
//
enum {
   ICP_RC_OK            = 0,
   ICP_RC_ILLEGAL       = 1,
   ICP_RC_FLASH_ERR     = 2,
   ICP_RC_VERIFY_ERR    = 3,
   };

//! Commands for BDM when in ICP mode
//
enum {
   ICP_GET_RESULT  =  1,            //!< Get result of last command
   ICP_ERASE_PAGE  =  2,            //!< Erase page
   ICP_PROGRAM_ROW =  3,            //!< Program row
   ICP_VERIFY_ROW  =  4,            //!< Verify row
   ICP_REBOOT      =  5,            //!< Reboot
   ICP_GET_VER     = CMD_GET_VER,   //!< Get version - common to both modes
};

#endif
