/*! \file
    \brief Provides a Dialogue Window for configuring the BDM.

    Provides a Dialogue Window allowing the user to configure the BDM interface.\n
    The BDM is configured on closing the dialogue.
   \verbatim
   Change History
   -=========================================================================
   | 27 Dec 2008 | Added extendedConnect() and associated changes       - pgo
   +=========================================================================
   \endverbatim
*/

#include <windows.h>
#include <stdio.h>
#include "Common.h"
#include "commands.h"
#include "interface.h"
#include "Resource.h"
#include "Configure.h"
#include "common_usb.h"
#include "log.h"

typedef enum {
   HAS_CLK_SW      = (1<<0),
   HAS_CLK_TRIM    = (1<<1),
   HAS_GUESS_SPEED = (1<<2),
   HAS_USE_RESET   = (1<<3),
} TargetOptions;

static LRESULT CALLBACK Configure(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

static unsigned int targetProperties  = 0;         //!< Which dialogue properties are supported on the target

// Various Target options represented on the Dialogue
static enum TargetVddSelection targetVdd     = BDM_TARGET_VDD_OFF; //!< Target Vdd choice
static enum TargetVddSelection lastTargetVdd = BDM_TARGET_VDD_3V3; //!< Last Target Vdd choice
static int enableCycleVddOnReset             = TRUE;               //!< Cycle Target Vdd on Reset (only if TargetVddSelection != OFF)
static int enableCycleVddOnConnect           = TRUE;               //!< Cycle Target Vdd on Connection problems (only if TargetVddSelection != OFF)
static int enableManuallyCycleVdd            = TRUE;               //!< Prompt user to cycle Target Vdd on Connection problems (only if TargetVddSelection == OFF)
static int enableLeaveTargetPowered          = FALSE;              //!< Leave target power on exit from the debugger (only if TargetVddSelection != OFF)
static int enableAutomaticReconnect          = FALSE;              //!< Automatically reconnect to target (handles speed changes)
static int enableGuessSpeed                  = TRUE;               //!< Guess speed (on early HC12 processors not supporting SYNC)
static enum ClkSwValues clkSwValue           = DEFAULT;            //!< ClkSW value
static enum ClkSwValues lastClkSwValue       = NORMAL;             //!< Last ClkSW value
static int enableUseResetSignal              = TRUE;               //!< Use reset signal on HCS08/Coldfire targets
static unsigned long trimFreq                = 0UL;                //!< Frequency to trim RS08 target clock to (0=>preserve)
static unsigned long lastTrimFreq            = 10000000UL;         //!< Last Frequency to trim RS08 target clock to (0=>preserve)
static int rs08Derivative                    = 0;                  //!< RS08 derivative (KA1 etc)

static int capabilities       = 0;         //!< Capability vector returned from BDM interface
static int target_type        = T_OFF;     //!< Target type (HC12 etc)

//! Scratch buffer for USB commands
static unsigned char usb_data[10];

//! \brief Passes a set option command to the BDM
//!
//! @param option Option number
//! @param value  Value for the option
//!
static void bdm_set_option(unsigned char option, unsigned int value) {
   usb_data[0] = 2;  // receive up to 2 bytes
   usb_data[1] = CMD_OPTION;
   usb_data[2] = option;
   usb_data[3] = (unsigned char) value&0xFF;
   usb_data[4] = (unsigned char) (value>>8)&0xFF;
   bdm_usb_recv_ep0(usb_data);
}

//! \brief Obtains the Capability vector from the BDM interface
//!
//! @return Capability vector
//!
static unsigned int bdm_get_capabilities(void) {
   usb_data[0] = 3;  // receive up to 3 bytes
   usb_data[1] = CMD_OPTION;
   usb_data[2] = BDM_GET_CAPABILITIES;
   if (bdm_usb_recv_ep0(usb_data) != 0)
      return 0; // Assume no capabilities on error
   return ((usb_data[2]<<8)+usb_data[1]);
}

//! \brief Sets RS08 target derivative type
//!
//! @param derivative Derivative number, see \ref rs08_derivative_type_e
//!
static void set_derivative_type(unsigned char derivative) {

   usb_data[0] = 1;  // receive up to 1 byte
   usb_data[1] = 0;//CMD_SET_DERIVATIVE;
   usb_data[2] = derivative;
   bdm_usb_recv_ep0(usb_data);
}

//! \brief Configure the BDM based upon the values entered in the configuration dialogue
//!
static void configureBDM( void ) {

   U16 trimSync;

   bdm_set_option(BDM_OPT_SET_TARGET_VDD,  targetVdd);

   if (targetVdd == BDM_TARGET_VDD_OFF) {
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_RESET,     FALSE);
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT,   FALSE);
      bdm_set_option(BDM_OPT_LEAVE_TARGET_POWERED,          FALSE);
   }
   else {
      enableManuallyCycleVdd = FALSE;
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_RESET,     enableCycleVddOnReset);
      bdm_set_option(BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT,   enableCycleVddOnConnect);
      bdm_set_option(BDM_OPT_LEAVE_TARGET_POWERED,          enableLeaveTargetPowered);
   }

   bdm_set_option(BDM_OPT_AUTOMATIC_RECONNECT,        enableAutomaticReconnect);
   bdm_set_option(BDM_OPT_GUESS_SPEED,                enableGuessSpeed);

   bdm_set_option(BDM_OPT_USE_ALT_BDM_CLOCK,          clkSwValue);

   bdm_set_option(BDM_OPT_USE_RESET_SIGNAL,           enableUseResetSignal);

   if (trimFreq==0)
      bdm_set_option(BDM_OPT_SET_CLOCK_TRIM_FREQ,      0);
   else {
      trimSync = (U16)(2 * 128 * (60000000.0/trimFreq));
      bdm_set_option(BDM_OPT_SET_CLOCK_TRIM_FREQ,      trimSync);
   }

   switch (rs08Derivative) {
      case 0: // KA1
         set_derivative_type(0);
         break;
      case 1: // KA2
         set_derivative_type(1);
         break;
      case 2: // KA4
         set_derivative_type(4);
         break;
      case 3: // KA8
         set_derivative_type(5);
         break;
      default:
         set_derivative_type(0);
         break;
   }
}

//! \brief Set the dialogue internal state to the default
//!
//! @param hDlg Window Handle for the dialogue
//!
static void setToDefault(HWND hDlg) {
   char buffer[10];

   // Set options to default
   // updateDialog will validate these for the particular dialog being used.
   targetVdd                  = BDM_TARGET_VDD_OFF;
   lastTargetVdd              = BDM_TARGET_VDD_3V3;
   enableCycleVddOnReset      = FALSE;
   enableCycleVddOnConnect    = TRUE;
   enableLeaveTargetPowered   = FALSE;
   enableManuallyCycleVdd     = TRUE;
   enableAutomaticReconnect   = TRUE;
   enableGuessSpeed           = TRUE;
   clkSwValue                 = DEFAULT;
   lastClkSwValue             = NORMAL;
   enableUseResetSignal       = FALSE;
   trimFreq                   = 0UL;            // Preserve Trim value
   lastTrimFreq               = 10000000UL;
   rs08Derivative             = 0;

   //  The following are driven by the value in the dialogue
   ltoa(lastTrimFreq, buffer, 10 );
   SetDlgItemText(hDlg, IDC_CLOCK_FREQ_EDIT, buffer);

   if (target_type == T_RS08) {
      HWND rs08DerivativeList      = GetDlgItem(hDlg, IDC_RS08_DERIVATIVE_LIST);
      SendMessage(rs08DerivativeList, CB_GETCURSEL, (WPARAM)0, (LPARAM)0);
   }
}

//! \brief Update the dialogue from internal state
//!
//! @param hDlg Window Handle for the dialogue
//!
static void updateDialog(HWND hDlg) {
   HWND VddEnableEn              = GetDlgItem(hDlg, IDC_VDD_CHECKBOX);
   HWND radio3V3En               = GetDlgItem(hDlg, IDC_3V3_RADIO);
   HWND radio5VEn                = GetDlgItem(hDlg, IDC_5V_RADIO);
   HWND checkVddCycleOnReset     = GetDlgItem(hDlg, IDC_CYCLE_VDD_ON_RESET_CHECKBOX);
   HWND checkVddCycleOnConnect   = GetDlgItem(hDlg, IDC_CYCLE_VDD_ON_CONNECTION_CHECKBOX);
   HWND checkVddLeaveOn          = GetDlgItem(hDlg, IDC_LEAVE_TARGET_ON_CHECKBOX);
   HWND checkVddManuallyCycle    = GetDlgItem(hDlg, IDC_MANUALLY_CYCLE_VDD);
   HWND checkUseResetSignal      = GetDlgItem(hDlg, IDC_USE_RESET_CHECKBOX);
   HWND checkPreserveTrim        = GetDlgItem(hDlg, IDC_PRESERVE_TRIM_CHECKBOX);
   HWND rs08TrimFrequency        = GetDlgItem(hDlg, IDC_CLOCK_FREQ_EDIT);
   HWND clkSw0                   = GetDlgItem(hDlg, IDC_CLKSW_ALT);
   HWND clkSw1                   = GetDlgItem(hDlg, IDC_CLKSW_NORMAL);

   if (!(capabilities & BDM_CAP_VDDCONTROL)) {
      // Vdd control options disabled
      targetVdd                = BDM_TARGET_VDD_OFF;
      enableCycleVddOnReset    = FALSE;
      enableLeaveTargetPowered = FALSE;
      enableCycleVddOnConnect  = FALSE;
      EnableWindow(VddEnableEn, FALSE );
   }

   if (!(capabilities & BDM_CAP_RESET)) {
      if (targetProperties&HAS_USE_RESET) {
         // Reset control options disabled
         EnableWindow(checkUseResetSignal, TRUE );
         CheckDlgButton(hDlg, IDC_USE_RESET_CHECKBOX, enableUseResetSignal);
      }
      else { //!(targetProperties&HAS_USE_RESET)
         // Use Reset option must be on
         enableUseResetSignal  = TRUE;
      }
   }

   if (targetProperties & HAS_GUESS_SPEED) {
      CheckDlgButton(hDlg, IDC_GUESS_SPEED_CHECKBOX, enableGuessSpeed);
   }
   else { //!(targetProperties & BDM_GUESS_SPEED)
      enableGuessSpeed = FALSE;
   }

   if (targetProperties&HAS_CLK_SW) {
      // Alternative clock available on this target
      CheckDlgButton(hDlg, IDC_USE_ALT_BDM_CLOCK_CHECKBOX, clkSwValue != DEFAULT );
      EnableWindow(clkSw0, clkSwValue != DEFAULT );
      EnableWindow(clkSw1, clkSwValue != DEFAULT );
      switch(lastClkSwValue) {
         case ALT:
            CheckRadioButton(hDlg, IDC_CLKSW_ALT, IDC_CLKSW_NORMAL, IDC_CLKSW_ALT);
            break;
         default:
            CheckRadioButton(hDlg, IDC_CLKSW_ALT, IDC_CLKSW_NORMAL, IDC_CLKSW_NORMAL);
            break;
      }
   }
   else { //!(targetProperties&HAS_CLK_SW)
      // No alternative clock
      clkSwValue     = DEFAULT;
      lastClkSwValue = DEFAULT;
   }

   if (targetProperties&HAS_CLK_TRIM) {
      // Allow trim value to be controlled
      EnableWindow(checkPreserveTrim,                    TRUE );
      EnableWindow(rs08TrimFrequency,                    trimFreq != 0);
      CheckDlgButton(hDlg, IDC_PRESERVE_TRIM_CHECKBOX,   trimFreq == 0);
   }
   else {
      // Only have trim value on RS08
      trimFreq = 0;
   }

   CheckDlgButton(hDlg, IDC_VDD_CHECKBOX, targetVdd!=BDM_TARGET_VDD_OFF);
   EnableWindow(radio3V3En,               targetVdd!=BDM_TARGET_VDD_OFF );
   EnableWindow(radio5VEn,                targetVdd!=BDM_TARGET_VDD_OFF );
   EnableWindow(checkVddCycleOnReset,     targetVdd!=BDM_TARGET_VDD_OFF );
   EnableWindow(checkVddCycleOnConnect,   targetVdd!=BDM_TARGET_VDD_OFF );
   EnableWindow(checkVddLeaveOn,          targetVdd!=BDM_TARGET_VDD_OFF );

   if (targetVdd == BDM_TARGET_VDD_OFF)
      EnableWindow(checkVddManuallyCycle, TRUE );
   else
      EnableWindow(checkVddManuallyCycle, FALSE );

   CheckDlgButton(hDlg, IDC_MANUALLY_CYCLE_VDD, enableManuallyCycleVdd);

   switch(lastTargetVdd){
      case BDM_TARGET_VDD_5V :
         CheckRadioButton(hDlg, IDC_3V3_RADIO, IDC_5V_RADIO, IDC_5V_RADIO);
         break;
      default :
         CheckRadioButton(hDlg, IDC_3V3_RADIO, IDC_5V_RADIO, IDC_3V3_RADIO);
         break;
   }

   CheckDlgButton(hDlg, IDC_CYCLE_VDD_ON_RESET_CHECKBOX,       enableCycleVddOnReset);
   CheckDlgButton(hDlg, IDC_CYCLE_VDD_ON_CONNECTION_CHECKBOX,  enableCycleVddOnConnect);
   CheckDlgButton(hDlg, IDC_LEAVE_TARGET_ON_CHECKBOX,          enableLeaveTargetPowered);

   CheckDlgButton(hDlg, IDC_RECONNECT_CHECKBOX,                enableAutomaticReconnect);
}

//! \brief Windows Callback to handle the dialogue events
//!
//! @param hDlg     Window Handle for the dialogue
//! @param message
//! @param wParam
//! @param lParam
//!
static LRESULT CALLBACK Configure(HWND hDlg, UINT message,
                                  WPARAM wParam, LPARAM lParam)
{
  // WM_COMMAND wNotifyCode = HIWORD(wParam);
  // wID = LOWORD(wParam);
  // hwndCtl = (HWND) lParam;
  char buffer[10];

   switch (message) {
      case WM_INITDIALOG:
         // Populate RS08 derivative list
         SendDlgItemMessage(hDlg, IDC_RS08_DERIVATIVE_LIST, CB_ADDSTRING, (WPARAM) 0, (LPARAM)"KA1");
         SendDlgItemMessage(hDlg, IDC_RS08_DERIVATIVE_LIST, CB_ADDSTRING, (WPARAM) 0, (LPARAM)"KA2");
         SendDlgItemMessage(hDlg, IDC_RS08_DERIVATIVE_LIST, CB_ADDSTRING, (WPARAM) 0, (LPARAM)"KA4");
         SendDlgItemMessage(hDlg, IDC_RS08_DERIVATIVE_LIST, CB_ADDSTRING, (WPARAM) 0, (LPARAM)"KA8");
         SendDlgItemMessage(hDlg, IDC_RS08_DERIVATIVE_LIST, CB_SETCURSEL, (WPARAM) 0, (LPARAM) 0);

         //SetFocus(hDlg);
         setToDefault(hDlg);
         updateDialog(hDlg);
         return TRUE;

      case WM_COMMAND:
         switch (LOWORD(wParam)) {
            case IDOK :
            case IDCANCEL :
               if (target_type == T_RS08) {
                  GetDlgItemText(hDlg, IDC_CLOCK_FREQ_EDIT, buffer, 10);
                  trimFreq = atol(buffer);
                  HWND rs08DerivativeList = GetDlgItem(hDlg, IDC_RS08_DERIVATIVE_LIST);
                  rs08Derivative = SendMessage(rs08DerivativeList, CB_GETCURSEL, (WPARAM)0, (LPARAM)0);
                  if (rs08Derivative<0)
                     rs08Derivative = 0;
                  print("Derivative = %d\r\n", rs08Derivative);
               }
               EndDialog(hDlg, LOWORD(wParam));
               return TRUE;
            case IDC_VDD_CHECKBOX :
               if (targetVdd == BDM_TARGET_VDD_OFF)
                  targetVdd = lastTargetVdd;
               else
                  targetVdd = BDM_TARGET_VDD_OFF;
               updateDialog(hDlg);
               break;
            case IDC_3V3_RADIO :
               targetVdd                 = BDM_TARGET_VDD_3V3;
               lastTargetVdd             = BDM_TARGET_VDD_3V3;
               updateDialog(hDlg);
               break;
            case IDC_5V_RADIO :
               targetVdd                 = BDM_TARGET_VDD_5V;
               lastTargetVdd             = BDM_TARGET_VDD_5V;
               updateDialog(hDlg);
               break;
            case IDC_CYCLE_VDD_ON_RESET_CHECKBOX :
               enableCycleVddOnReset     = !enableCycleVddOnReset;
               updateDialog(hDlg);
               break;
            case IDC_CYCLE_VDD_ON_CONNECTION_CHECKBOX :
               enableCycleVddOnConnect   = !enableCycleVddOnConnect;
               updateDialog(hDlg);
               break;
            case IDC_LEAVE_TARGET_ON_CHECKBOX :
               enableLeaveTargetPowered  = !enableLeaveTargetPowered;
               updateDialog(hDlg);
               break;
            case IDC_MANUALLY_CYCLE_VDD :
               enableManuallyCycleVdd  = !enableManuallyCycleVdd;
               updateDialog(hDlg);
               break;
            case IDC_RECONNECT_CHECKBOX :
               enableAutomaticReconnect  = !enableAutomaticReconnect;
               updateDialog(hDlg);
               break;
            case IDC_GUESS_SPEED_CHECKBOX :
               enableGuessSpeed            = !enableGuessSpeed;
               updateDialog(hDlg);
               break;
            case IDC_USE_ALT_BDM_CLOCK_CHECKBOX :
               if (clkSwValue == DEFAULT)
                  clkSwValue = lastClkSwValue;
               else
                  clkSwValue = DEFAULT;
               updateDialog(hDlg);
               break;
            case IDC_CLKSW_ALT :
               clkSwValue      = ALT;
               lastClkSwValue  = ALT;
               updateDialog(hDlg);
               break;
            case IDC_CLKSW_NORMAL :
               clkSwValue      = NORMAL;
               lastClkSwValue  = NORMAL;
               updateDialog(hDlg);
               break;
            case IDC_USE_RESET_CHECKBOX :
               enableUseResetSignal  = !enableUseResetSignal;
               updateDialog(hDlg);
               break;
            case IDC_PRESERVE_TRIM_CHECKBOX :
               if (trimFreq == 0)
                  trimFreq = lastTrimFreq;
               else {
                  GetDlgItemText(hDlg, IDC_CLOCK_FREQ_EDIT, buffer, 10);
                  lastTrimFreq = atol(buffer);
                  trimFreq = 0;
               }
               updateDialog(hDlg);
               break;
            case IDC_CLOCK_FREQ_EDIT :
               //GetDlgItemText(hDlg, IDC_CLOCK_FREQ_EDIT, buffer, 10);
               //trimFreq = atol(buffer);
               //updateDialog(hDlg);
               break;
            case IDC_RS08_DERIVATIVE_LIST :
               //if (HIWORD(wParam) == CBN_SELCHANGE)
               //   updateDialog(hDlg);
               break;
            case IDDEFAULT :
               setToDefault(hDlg);
               updateDialog(hDlg);
               break;
            default :
               return FALSE;
         }
   }
   return FALSE;
}

//ToDo
void loadSettingsFile(void) {
   char path[MAX_PATH];
   char *charp;
   FILE *configFile = NULL;

   GetModuleFileName(NULL, path, sizeof(path));
   charp=strchr(path,'\\');
   if (charp!=NULL) {
      while (strchr(charp+1,'\\'))
         charp=strchr(charp+1,'\\'); // Find the last backslash
      strcpy(charp+1,"usbdm.cfg");
   }

   configFile = fopen(path,"rt");

   if (configFile != NULL)
      fclose(configFile);
}

//ToDo
void saveSettingsFile(void) {
   char path[MAX_PATH];
   char *charp;
   FILE *configFile = NULL;

   GetModuleFileName(NULL,path,sizeof(path));
   charp=strchr(path,'\\');
   if (charp!=NULL) {
      while (strchr(charp+1,'\\'))
         charp=strchr(charp+1,'\\'); /* find the last backslash */
      strcpy(charp+1,"usbdm.cfg");
   }

   configFile=fopen(path,"wt");

   fprintf(configFile, "BDM_OPT_SET_TARGET_VDD=%d\n",              targetVdd);
   fprintf(configFile, "BDM_OPT_CYCLE_TARGET_VDD_ON_CONNECT=%d\n", enableCycleVddOnConnect);
   fprintf(configFile, "BDM_OPT_AUTOMATIC_RECONNECT=%d\n",         enableAutomaticReconnect);
   fprintf(configFile, "BDM_OPT_GUESS_SPEED=%d\n",                 enableGuessSpeed);
   fprintf(configFile, "BDM_OPT_LEAVE_TARGET_POWERED=%d\n",        enableLeaveTargetPowered);
   fprintf(configFile, "BDM_OPT_USE_ALT_BDM_CLOCK=%d\n",           clkSwValue);
   fprintf(configFile, "BDM_OPT_CYCLE_TARGET_VDD_ON_RESET=%d\n",   enableCycleVddOnReset);
   fprintf(configFile, "BDM_OPT_USE_RESET_SIGNAL=%d\n",            enableUseResetSignal);
   fprintf(configFile, "BDM_OPT_SET_CLOCK_TRIM_FREQ=%ld\n",        trimFreq);
   fclose(configFile);
}

//! \brief Display Configuration Dialogue.
//!
//!
//! The BDM interface is configured when the dialogue is closed.
//!
//! @param hDLLInst     DLL Handle (to obtain resources)
//! @param target       Target processor type so that the appropriate dialogue may be displayed.
//!
void configureDialog(HINSTANCE hDLLInst, target_type_e target){
int rc;

   print("configureDialog: hDLLInst = %d\r\n", hDLLInst);

   //loadSettingsFile();

   target_type = target;

   capabilities = bdm_get_capabilities();

   loadSettingsFile();

   switch (target) {
      case T_HC12:
         targetProperties = HAS_CLK_SW|HAS_GUESS_SPEED;
         rc = DialogBox(hDLLInst, (LPCTSTR)IDD_USBDM_CONFIGURE_HC12, 0, (DLGPROC)Configure);
         break;
      case T_HCS08:
         targetProperties = HAS_CLK_SW|HAS_USE_RESET;
         rc = DialogBox(hDLLInst, (LPCTSTR)IDD_USBDM_CONFIGURE_HC08, 0, (DLGPROC)Configure);
         break;
      case T_RS08:
         targetProperties = HAS_CLK_TRIM|HAS_USE_RESET;
         rc = DialogBox(hDLLInst, (LPCTSTR)IDD_USBDM_CONFIGURE_RS08, 0, (DLGPROC)Configure);
         break;
      case T_CFV1:
         targetProperties = HAS_CLK_SW|HAS_USE_RESET;
         rc = DialogBox(hDLLInst, (LPCTSTR)IDD_USBDM_CONFIGURE_CFV1, 0, (DLGPROC)Configure);
         break;
      default:
         return;
   }

   if (rc <0) { // Dialogue failure
      LPVOID lpMsgBuf;
      DWORD dw = GetLastError();

      print("configureDialog: rc = %d\r\n", rc);

      FormatMessage(
          FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |FORMAT_MESSAGE_IGNORE_INSERTS,
          NULL,
          dw,
          MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
          (LPTSTR) &lpMsgBuf,
          0, NULL );
      print("configureDialog: Error %s", lpMsgBuf);
   }

   saveSettingsFile();

   configureBDM();   // Configure BDM
}

//==========================================================================
//
//

//! Connects to Target.
//!
//! This will cause the BDM module to attempt to connect to the Target.
//! In most cases the BDM module will automatically determine the connection
//! speed and successfully connect.
//!
//! @return 0 => Success,\n !=0 => Fail
//!
unsigned char simple_target_sync(void) {
   usb_data[0] = 1;  // receive up to 1 byte
   usb_data[1] = 0;//CMD_CONNECT;
   bdm_usb_recv_ep0(usb_data);
   return usb_data[0]!=0;//CMD_CONNECT;
}

//! \brief Windows Callback to handle the dialogue events
//!
//! @param hDlg     Window Handle for the dialogue
//! @param message
//! @param wParam
//! @param lParam
//!
static LRESULT CALLBACK powerCycleDialogHandler(HWND hDlg, UINT message,
                                                WPARAM wParam, LPARAM lParam)
{
  // WM_COMMAND wNotifyCode = HIWORD(wParam);
  // wID = LOWORD(wParam);
  // hwndCtl = (HWND) lParam;
   switch (message) {
      case WM_INITDIALOG:
         //SetFocus(hDlg);
         return TRUE;

      case WM_COMMAND:
         switch (LOWORD(wParam)) {
            case IDOK :
            case IDCANCEL :
               EndDialog(hDlg, LOWORD(wParam));
               return TRUE;
            default :
               return FALSE;
         }
   }
   return FALSE;
}

//! Extended process for connecting to the target
//! This procedure will prompt the user to
//! cycle the target power supply if necessary.  The
//! BKGD & RESET lines will be held low while this is occurring.
//!
//! @param hDLLInst     DLL Handle (to obtain resources)
//!
unsigned char extendedConnect(HINSTANCE hDLLInst){
int rc,rc2;
int giveUp = FALSE;

   print("extendedConnect() - 1st attempt\r\n");
   rc = simple_target_sync();

   while ((rc != 0) && enableManuallyCycleVdd && !giveUp) {
      print("extendedConnect() - retry\r\n");
      bdm_set_option(BDM_OPT_SET_INTERFACELEVEL, 0); // Set BKGD & RESET low

      // Prompt user to cycle power
      rc2 = DialogBox(hDLLInst, (LPCTSTR)IDD_CYCLE_POWER, 0, (DLGPROC)powerCycleDialogHandler);

      if (rc2 <0) { // Dialogue failure
         LPVOID lpMsgBuf;
         DWORD dw = GetLastError();

         print("extendedConnect(): rc2 = %d\r\n", rc2);

         FormatMessage(
             FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
             NULL,
             dw,
             MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
             (LPTSTR) &lpMsgBuf,
             0, NULL );
         print("extendedConnect(): Dialogue Error %s", lpMsgBuf);
         break;
      }

      switch (rc2) {
         case IDOK :  // User has cycled power - try connect
            print("extendedConnect() - retry\r\n");
            bdm_set_option(BDM_OPT_SET_INTERFACELEVEL, 0x03); // Release BKGD & RESET
            Sleep(200 /* ms */); // Give target time
            rc = simple_target_sync();
            break;
         default :  // Assume user has given up
            print("extendedConnect() - no further retries\r\n");
            giveUp = TRUE;
            break;
      }
      bdm_set_option(BDM_OPT_SET_INTERFACELEVEL, 0x03); // Release BKGD & RESET
   }

   return rc;
}

