/*! \file
    \brief Header file for TBLCF routines

 */
#ifndef _TBLCF_H_
#define _TBLCF_H_

#if defined(DLL)
//! This definition is used when being exported
#define USBDM_API WINAPI __declspec(dllexport)
#elif defined(COMMANDLINE)
//! This definition is used when being directly linked to
#define USBDM_API
#else
//! This definition is used when being imported
#define USBDM_API WINAPI __declspec(dllimport)
#endif

// This macro adds the correct prefix to DLL entry point
#if defined(TBLCF)
#define prefix(x) tblcf_ ## x
#elif defined(TBDML)
#define prefix(x) tbdml_ ## x
#else
#define prefix(x) opensourcebdm_ ## x
#endif

#define WRITE_BLOCK_CHECK

//===================================================================================
//!  Target microcontroller types
typedef enum {
   T_HC12      = 0,     //!< HC12 or HCS12 target
   T_HCS08     = 1,     //!< HCS0-8 target
   T_RS08      = 2,     //!< RS08 target
   T_CFV1      = 3,     //!< Coldfire Version 1 target
   T_CFVx      = 4,     //!< Coldfire Version 2,3,4 target
   T_JTAG      = 5,     //!< JTAG target
   T_EZFLASH   = 6,     //!< EzPort Flash interface (SPI?)
   T_OFF       = 15,    //!< Turn off interface (no target)
} target_type_e;

//!  Reset mode (normal operation or BDM)
typedef enum {	/* type of reset action required */
	BDM_MODE=0, //!< Special mode (BDM active, Target halted)
	NORMAL_MODE  = 1   //!< Normal mode (usual reset, Target executes)
} target_mode_e;

typedef enum {	/* target reset detection state */
	RESET_NOT_DETECTED=0,
	RESET_DETECTED=1
} reset_detection_e;

typedef enum {	/* target reset state */
	RSTO_ACTIVE=0,
	RSTO_INACTIVE=1
} reset_state_e;

typedef struct { /* state of BDM communication */
	reset_state_e reset_state;
	reset_detection_e reset_detection;
} bdmcf_status_t;

USBDM_API unsigned char  prefix(dll_version)(void); 				/* returns DLL version in BCD */
USBDM_API unsigned char  prefix(init)(void);						/* initialises USB and returns number of devices found */
USBDM_API unsigned char  prefix(open)(unsigned char device_no); 	/* opens a device with given number (0...), returns 0 on success and 1 on error */
USBDM_API void           prefix(close)(void);								/* closes currently open device */
USBDM_API unsigned int   prefix(get_version)(void);				/* returns hardware & software version of the cable in BCD format - SW version in lower byte and HW version in upper byte */
USBDM_API unsigned char  prefix(get_last_sts)(void); 				/* returns status of the last command: 0 on sucess and non-zero on failure */

USBDM_API unsigned char  prefix(set_target_type)(target_type_e target_type); 	/* sets target MCU type; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(resynchronize)(void); 			/* resynchronizes communication with the target (in case of noise, etc.); returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(target_reset)(target_mode_e target_mode); 	/* resets the target to normal or BDM mode; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(bdm_sts)(bdmcf_status_t *bdmcf_status); 		/* fills user supplied structure with current state of the BDM communication channel; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(target_go)(void); 				/* starts target execution from current PC address; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(target_step)(void);				/* steps over a single target instruction; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(target_halt)(void);				/* brings the target into BDM mode; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(assert_ta)(unsigned char duration_10us); 		/* asserts the TA signal for the specified time (in 10us ticks); returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(read_reg)(unsigned char reg_index, unsigned long int * result);	/* reads the specified register and writes its contents into the supplied buffer; returns 0 on success and non-zero on failure */
USBDM_API void           prefix(write_reg)(unsigned char reg_index, unsigned long int value); 				/* writes specified register */
USBDM_API unsigned char  prefix(read_creg)(unsigned int address, unsigned long int * result);		/* reads control register at the specified address and writes its contents into the supplied buffer; returns 0 on success and non-zero on failure */
USBDM_API void           prefix(write_creg)(unsigned int address, unsigned long int value);				/* writes control register at the specified address; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(read_dreg)(unsigned char dreg_index, unsigned long int * result); /* reads the specified debug register and writes its contents into the supplied buffer; returns 0 on success and non-zero on failure */
USBDM_API void           prefix(write_dreg)(unsigned char dreg_index, unsigned long int value); 			/* writes specified debug register */
USBDM_API unsigned char  prefix(read_mem8)(unsigned long int address, unsigned char * result); 	/* reads byte from the specified address; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(read_mem16)(unsigned long int address, unsigned int * result);	/* reads word from the specified address; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(read_mem32)(unsigned long int address, unsigned long int * result); /* reads long word from the specified address; returns 0 on success and non-zero on failure */
USBDM_API void           prefix(write_mem8)(unsigned long int address, unsigned char value);				/* writes byte at the specified address */
USBDM_API void           prefix(write_mem16)(unsigned long int address, unsigned int value); 				/* writes word at the specified address */
USBDM_API void           prefix(write_mem32)(unsigned long int address, unsigned long int value);			/* writes long word at the specified address */
USBDM_API unsigned char  prefix(read_block8)(unsigned long int address, unsigned long int bytecount, unsigned char *buffer);		/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer; uses byte accesses only; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(read_block16)(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer; uses word accesses; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(read_block32)(unsigned long int address, unsigned long int bytecount, unsigned char *buffer);		/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer; uses long word accesses; returns 0 on success and non-zero on failure */
USBDM_API unsigned char  prefix(write_block8)(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* writes the requested number of bytes to target memory from the supplied address; uses byte accesses only; returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
USBDM_API unsigned char  prefix(write_block16)(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* writes the requested number of bytes to target memory at the supplied address; uses word accesses; returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
USBDM_API unsigned char  prefix(write_block32)(unsigned long int address, unsigned long int bytecount, unsigned char *buffer); 	/* writes the requested number of bytes to target memory at the supplied address; uses long word accesses; returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */

USBDM_API unsigned char  prefix(jtag_sel_shift)(unsigned char mode);										/* JTAG - go from RUN-TEST/IDLE to SHIFT-DR (mode==0) or SHIFT-IR (mode!=0) state */
USBDM_API unsigned char  prefix(jtag_sel_reset)(void);													/* JTAG - go from RUN-TEST/IDLE to TEST-LOGIC-RESET state */
USBDM_API void           prefix(jtag_write)(unsigned char bit_count, unsigned char exit, unsigned char *buffer);	/* JTAG - write data; parameter exit: ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when done; data: shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte */
USBDM_API unsigned char  prefix(jtag_read)(unsigned char bit_count, unsigned char exit, unsigned char *buffer);	/* JTAG - read data; parameter exit: ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when done; data: shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte */

USBDM_API void           prefix(setBoot)(void);

USBDM_API void           prefix(icp_reboot)( void );
USBDM_API unsigned int   prefix(icp_program)(unsigned int addr, unsigned int count, unsigned char *data);
USBDM_API unsigned int   prefix(icp_verify)(unsigned int addr, unsigned int count, unsigned char *data);
USBDM_API unsigned int   prefix(icp_erase)(unsigned int addr, unsigned int count);
USBDM_API unsigned int   prefix(icp_get_result)( void );

#if defined (COMMANDLINE)
void bdm_debug(char usb_data[]);
void dll_initialize(HINSTANCE);
void dll_uninitialize(void);
#endif

#endif // _TBLCF_H_
