/*! \file
    \brief Entry points for opensourcebdm.dll library

    \verbatim
    Turbo BDM Light ColdFire - interface DLL
    Copyright (C) 2006 Daniel Malik

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
\endverbatim
*/

#include <windows.h>
#include <stdio.h>
#include "log.h"
#include "version.h"
#include "interface.h"
#include "Common.h"
#include "ICP.h"
#include "common_usb.h"
#include "commands.h"
#include "Configure.h"

/*
 *  Define symbol "LOG" in "log.h" during compilation to produce a log file tbdml_dll.log
 */

static unsigned char usb_data[MAX_PACKET_SIZE+1]; // Extra +1 for debug byte
static HINSTANCE hDLLInst;

//! Get version of the DLL in BCD format
//!
//! @return 8-bit version number (2 BCD digits, major-minor)
//!
USBDM_API unsigned char prefix(dll_version)(void) {
	return(TBLCF_DLL_VERSION);
}

//! Initialises USB interface
//!
//! This must be done before any device may be opened.
//!
//! @return Number of BDM devices found
//!
USBDM_API unsigned char prefix(init)(void) {
	unsigned char i;
	bdm_usb_find_devices();	/* look for devices on all USB busses */
	i=bdm_usb_cnt();
	print("TBLCF_INIT: Usb initialised, found %d device(s)\r\n",i);
	return(i);							/* count the devices found and return the number */
}

//! Opens a device
//!
//! @param device_no Number (0..N) of device to open.
//! A device must be open before any communication with the device can take place.
//! @return 0 => Success,\n !=0 => Fail
//!
USBDM_API unsigned char prefix(open)(unsigned char device_no) {
int rc = 0;

   print("OSBDM_OPEN: Trying to open device #%d\r\n",device_no);
   rc = bdm_usb_open(device_no);
   print("OSBDM_OPEN: rc = #%d\r\n", rc);
   if ((rc == 0) & (bdm_usb_cnt() == 1)) {
     // Successful open and only one device available
     checkFirmware(hDLLInst); // Check Firmware version
   }
   return (rc);
}

//! Closes currently open device
//!
USBDM_API void prefix(close)(void) {
	print("TBLCF_CLOSE: Trying to close the device\r\n");

	// Tell the BDM that we're finished (power down)
   usb_data[0]=2;  /* receive 2 byte */
   usb_data[1]=CMD_SET_TARGET;
   usb_data[2]=0xFF;
   bdm_usb_recv_ep0(usb_data);
	bdm_usb_close();
}

//! Gets software version and type of hardware
//!
//! @return 32-bit version number (4 bytes)
//!  - [0] = BDM software version                \n
//!  - [1] = hardware type                       \n
//!  - [2] = ICP software version                \n
//!  - [3] = hardware type (from ICP flash area) \n
//!  Bytes [1] and [3] should always agree
//!
USBDM_API unsigned int prefix(get_version)(void) {
   usb_data[0] = 5;  // receive up to 5 bytes
	usb_data[1]=CMD_GET_VER;
	bdm_usb_recv_ep0(usb_data);
	return (*((U32 *)(usb_data+1)));
}

//! Get status of the last command
//!
//! @return 0 => Success,\n 1 => Failure
//!
USBDM_API unsigned char prefix(get_last_sts)(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_GET_LAST_STATUS;
	bdm_usb_recv_ep0(usb_data);
	return ((usb_data[0]==CMD_FAILED)||(usb_data[0]==CMD_UNKNOWN));
}

//! Sets target MCU type
//!
//! @param target_type type of target, see \ref target_type_e
//!
//! @return 0 => Success,\n !=0 => Fail
//!
USBDM_API unsigned char prefix(set_target_type)(target_type_e target_type) {

   configureDialog(hDLLInst, target_type); // Do configuration dialogue

   usb_data[0] = 1;  // receive up to 1 byte
	usb_data[1]=CMD_SET_TARGET;
	usb_data[2]=target_type;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_SET_TARGET);
}

//! Resets the target to normal or special mode
//!
//! @param target_mode see \ref target_mode_e
//!
//! @return 0 => Success,\n !=0 => Fail
//!
USBDM_API unsigned char prefix(target_reset)(target_mode_e target_mode) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_RESET;
	usb_data[2]=target_mode;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_RESET);
}

//! Fills user supplied structure with state of BDM communication channel
//!
//! @param bdm_status Pointer to structure to receive status, see \ref bdm_status_t
//!
//! @return 0 => Success,\n !=0 => Fail
//!
USBDM_API unsigned char prefix(bdm_sts)(bdmcf_status_t *bdmcf_status) {
	usb_data[0]=3;	 /* get 3 bytes */
	usb_data[1]=CMD_GET_STATUS;
	bdm_usb_recv_ep0(usb_data);
	if (usb_data[0]!=CMD_GET_STATUS)
	   return (1);
	bdmcf_status->reset_state = ((usb_data[1]*256+usb_data[2])&RSTO_STATE_MASK)?RSTO_INACTIVE:RSTO_ACTIVE;
	bdmcf_status->reset_detection = ((usb_data[1]*256+usb_data[2])&RESET_DETECTED_MASK)?RESET_DETECTED:RESET_NOT_DETECTED;
	print("TBLCF_BDM_STATUS: Reported communication status 0x%04X (0x%02X)\r\n",(usb_data[1]*256+usb_data[2]),usb_data[0]);
	return(0);
}

//! Starts target execution from current PC address
//!
//! @return 0 => Success,\n !=0 => Fail
//!
USBDM_API unsigned char prefix(target_go)(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_GO;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_GO);
}

//! Steps over a single target instruction
//!
//! @return 0 => Success,\n !=0 => Fail
//!
USBDM_API unsigned char prefix(target_step)(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_STEP;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_STEP);
}

//! Brings the target into active background mode.  The target will be halted.
//!
//! @return 0 => Success,\n !=0 => Fail
//!
USBDM_API unsigned char prefix(target_halt)(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_HALT;
	bdm_usb_recv_ep0(usb_data);
	return (usb_data[0]!=CMD_HALT);
}


/* resynchronizes communication with the target (in case of noise, etc.) */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(resynchronize)(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_RESYNCHRONIZE;
	bdm_usb_recv_ep0(usb_data);
	print("TBLCF_RESYNCHRONIZE: (0x%02X)\r\n",usb_data[0]);
	return(!(usb_data[0]==CMD_RESYNCHRONIZE));
}

/* asserts the TA signal for the specified time (in 10us ticks) */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(assert_ta)(unsigned char duration_10us) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_ASSERT_TA;
	usb_data[2]=duration_10us;
	bdm_usb_recv_ep0(usb_data);
	print("TBLCF_ASSERT_TA: duration %d us (0x%02X)\r\n",(int)10*usb_data[2],usb_data[0]);
	return(!(usb_data[0]==CMD_ASSERT_TA));
}

/* reads control register at the specified address and writes its contents into the supplied buffer */
/* returns 0 on success and non-zero on failure */
//! (CFv1) Read Control register
//!
//! @param regNo  Register #
//!
//! @return 32-bit number
//!
USBDM_API unsigned char prefix(read_creg)(unsigned int address, unsigned long int * result) {
	usb_data[0]=5;	 /* get 5 bytes */
	usb_data[1]=CMD_READ_CREG;
	usb_data[2]=(address>>8)&0xff;	/* address in big endian */
	usb_data[3]=address&0xff;
	bdm_usb_recv_ep0(usb_data);
	*result = (((unsigned long int)usb_data[1])<<24)+(usb_data[2]<<16)+(usb_data[3]<<8)+usb_data[4];	/* result is in big endian */
	print("TBLCF_READ_CREG: Read control register from address 0x%04X, result: 0x%08lX (0x%02X)\r\n",address,*result,usb_data[0]);
	return(!(usb_data[0]==CMD_READ_CREG));
}

/* writes control register at the specified address */
USBDM_API void prefix(write_creg)(unsigned int address, unsigned long int value) {
	usb_data[0]=7;	 /* send 7 bytes */
	usb_data[1]=CMD_WRITE_CREG;
	usb_data[2]=(address>>8)&0xff;	/* address in big endian */
	usb_data[3]=address&0xff;
	usb_data[4]=(value>>24)&0xff;
	usb_data[5]=(value>>16)&0xff;
	usb_data[6]=(value>>8)&0xff;
	usb_data[7]=(value)&0xff;
	print("TBLCF_WRITE_CREG: Write control register at address 0x%04X with 0x%08lX\r\n",address,value);
	bdm_usb_send_ep0(usb_data);
}

/* reads the specified debug register and writes its contents into the supplied buffer */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_dreg)(unsigned char dreg_index, unsigned long int * result) {
	usb_data[0]=5;	 /* get 5 bytes */
	usb_data[1]=CMD_READ_DREG;
	usb_data[2]=dreg_index;
	bdm_usb_recv_ep0(usb_data);
	*result = (((unsigned long int)usb_data[1])<<24)+(usb_data[2]<<16)+(usb_data[3]<<8)+usb_data[4];	/* result is in big endian */
	print("TBLCF_READ_DREG: Read debug register #0x%02X, result: 0x%08lX (0x%02X)\r\n",dreg_index,*result,usb_data[0]);
	return(!(usb_data[0]==CMD_READ_DREG));
}

/* writes specified debug register */
USBDM_API void prefix(write_dreg)(unsigned char dreg_index, unsigned long int value) {
	usb_data[0]=6;	 /* send 6 bytes */
	usb_data[1]=CMD_WRITE_DREG;
	usb_data[2]=dreg_index&0xff;
	usb_data[3]=(value>>24)&0xff;
	usb_data[4]=(value>>16)&0xff;
	usb_data[5]=(value>>8)&0xff;
	usb_data[6]=(value)&0xff;
	print("TBLCF_WRITE_DREG: Write debug register #0x%02X with 0x%08lX\r\n",dreg_index,value);
	bdm_usb_send_ep0(usb_data);
}

/* reads the specified register and writes its contents into the supplied buffer */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_reg)(unsigned char reg_index, unsigned long int * result) {
	usb_data[0]=5;	 /* get 5 bytes */
	usb_data[1]=CMD_READ_REG;
	usb_data[2]=reg_index;
	bdm_usb_recv_ep0(usb_data);
	*result = (((unsigned long int)usb_data[1])<<24)+(usb_data[2]<<16)+(usb_data[3]<<8)+usb_data[4];	/* result is in big endian */
	print("TBLCF_READ_REG: Read register #0x%02X, result: 0x%08lX (0x%02X)\r\n",reg_index,*result,usb_data[0]);
	return(!(usb_data[0]==CMD_READ_REG));
}

/* writes specified register */
USBDM_API void prefix(write_reg)(unsigned char reg_index, unsigned long int value) {
	usb_data[0]=6;	 /* send 6 bytes */
	usb_data[1]=CMD_WRITE_REG;
	usb_data[2]=reg_index&0xff;
	usb_data[3]=(value>>24)&0xff;
	usb_data[4]=(value>>16)&0xff;
	usb_data[5]=(value>>8)&0xff;
	usb_data[6]=(value)&0xff;
	print("TBLCF_WRITE_REG: Write register #0x%02X with 0x%08lX\r\n",reg_index,value);
	bdm_usb_send_ep0(usb_data);
}

/* reads byte from the specified address */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_mem8)(unsigned long int address, unsigned char * result) {
	usb_data[0]=2;	 /* get 2 bytes */
	usb_data[1]=CMD_READ_MEM8;
	usb_data[2]=(address>>24)&0xff;
	usb_data[3]=(address>>16)&0xff;
	usb_data[4]=(address>>8)&0xff;
	usb_data[5]=(address)&0xff;
	bdm_usb_recv_ep0(usb_data);
	*result = usb_data[1];
	print("TBLCF_READ_MEM8: Read byte from address 0x%08lX, result: 0x%02X (0x%02X)\r\n",address,*result,usb_data[0]);
	return(!(usb_data[0]==CMD_READ_MEM8));
}

/* reads word from the specified address */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_mem16)(unsigned long int address, unsigned int * result) {
	usb_data[0]=3;	 /* get 3 bytes */
	usb_data[1]=CMD_READ_MEM16;
	usb_data[2]=(address>>24)&0xff;
	usb_data[3]=(address>>16)&0xff;
	usb_data[4]=(address>>8)&0xff;
	usb_data[5]=(address)&0xff;
	bdm_usb_recv_ep0(usb_data);
	*result = ((unsigned int)usb_data[1]<<8)+usb_data[2];
	print("TBLCF_READ_MEM16: Read word from address 0x%08lX, result: 0x%04X (0x%02X)\r\n",address,*result,usb_data[0]);
	return(!(usb_data[0]==CMD_READ_MEM16));
}

/* reads long word from the specified address */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_mem32)(unsigned long int address, unsigned long int * result) {
	usb_data[0]=5;	 /* get 5 bytes */
	usb_data[1]=CMD_READ_MEM32;
	usb_data[2]=(address>>24)&0xff;
	usb_data[3]=(address>>16)&0xff;
	usb_data[4]=(address>>8)&0xff;
	usb_data[5]=(address)&0xff;
	bdm_usb_recv_ep0(usb_data);
	*result = (((unsigned long int)usb_data[1])<<24)+(usb_data[2]<<16)+(usb_data[3]<<8)+usb_data[4];
	print("TBLCF_READ_MEM32: Read long word from address 0x%08lX, result: 0x%08lX (0x%02X)\r\n",address,*result,usb_data[0]);
	return(!(usb_data[0]==CMD_READ_MEM32));
}

/* writes byte at the specified address */
USBDM_API void prefix(write_mem8)(unsigned long int address, unsigned char value) {
	usb_data[0]=6;	 /* send 6 bytes */
	usb_data[1]=CMD_WRITE_MEM8;
	usb_data[2]=(address>>24)&0xff;
	usb_data[3]=(address>>16)&0xff;
	usb_data[4]=(address>>8)&0xff;
	usb_data[5]=(address)&0xff;
	usb_data[6]=(value)&0xff;
	print("TBLCF_WRITE_MEM8: Write byte 0x%02X to address 0x%08lX\r\n",value,address);
	bdm_usb_send_ep0(usb_data);
}

/* writes word at the specified address */
USBDM_API void prefix(write_mem16)(unsigned long int address, unsigned int value) {
	usb_data[0]=7;	 /* send 7 bytes */
	usb_data[1]=CMD_WRITE_MEM16;
	usb_data[2]=(address>>24)&0xff;
	usb_data[3]=(address>>16)&0xff;
	usb_data[4]=(address>>8)&0xff;
	usb_data[5]=(address)&0xff;
	usb_data[6]=(value>>8)&0xff;
	usb_data[7]=(value)&0xff;
	print("TBLCF_WRITE_MEM16: Write word 0x%04X to address 0x%08lX\r\n",value,address);
	bdm_usb_send_ep0(usb_data);
}

/* writes long word at the specified address */
USBDM_API void prefix(write_mem32)(unsigned long int address, unsigned long int value) {
	usb_data[0]=9;	 /* send 9 bytes */
	usb_data[1]=CMD_WRITE_MEM32;
	usb_data[2]=(address>>24)&0xff;
	usb_data[3]=(address>>16)&0xff;
	usb_data[4]=(address>>8)&0xff;
	usb_data[5]=(address)&0xff;
	usb_data[6]=(value>>24)&0xff;
	usb_data[7]=(value>>16)&0xff;
	usb_data[8]=(value>>8)&0xff;
	usb_data[9]=(value)&0xff;
	print("TBLCF_WRITE_MEM32: Write long word 0x%08lX to address 0x%08lX\r\n",value,address);
	bdm_usb_send_ep0(usb_data);
}

/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer */
/* uses byte accesses only */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_block8)( unsigned long int address,
                                           unsigned long int byteCount,
                                           unsigned char *buffer) {
   const unsigned maxBlockSize = (MAX_DATA_SIZE-1);
	unsigned int   blockSize;

	print("TBLCF_READ_BLOCK8: Read 0x%08lX byte(s) from address 0x%08lX:\r\n",
	      byteCount, address );

	while (byteCount > 0) {
      blockSize = byteCount;
      if (blockSize > maxBlockSize)
         blockSize = maxBlockSize;
		usb_data[0] = blockSize+1;
		usb_data[1] = CMD_READ_MEMBLOCK8;
		usb_data[2] = (address>>24)&0xff;
		usb_data[3] = (address>>16)&0xff;
		usb_data[4] = (address>>8)&0xff;
		usb_data[5] = (address)&0xff;
		bdm_usb_recv_ep0(usb_data);
		print("TBLCF_READ_BLOCK8: Block read, size 0x%02X (0x%02X):\r\n",
		      blockSize, usb_data[0]);
		if (usb_data[0] != CMD_READ_MEMBLOCK8)
         return(1);
      memcpy(buffer, usb_data+1, blockSize);
		printDump(buffer, blockSize);
		byteCount -= blockSize;
      address   += blockSize;
      buffer    += blockSize;
	}
	return(0);
}

/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer */
/* uses word accesses */
/* if the address is odd a byte read is performed to make it even before performing the word reads */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_block16)( unsigned long int address,
                                            unsigned long int byteCount,
                                            unsigned char *buffer) {
	const unsigned maxBlockSize = ((MAX_DATA_SIZE-1)&0xfffe);
   unsigned int   blockSize;

	print( "TBLCF_READ_BLOCK16: Read 0x%08lX byte(s) from address 0x%08lX:\r\n",
	       byteCount, address);

	if (address&0x01) { // Odd address - read 1 byte to make even
		print("TBLCF_READ_BLOCK16: Address is odd, performing 1 byte read\r\n");
		if (prefix(read_mem8)(address, buffer))
		   return(1);
		byteCount--;
		address++;
		buffer++;
	}
	while (byteCount >= 2) { // Read words
      blockSize = byteCount;
      if (blockSize > maxBlockSize)
         blockSize = maxBlockSize;
		usb_data[0] = blockSize+1;	 /* receive block of maximum size */
		usb_data[1] = CMD_READ_MEMBLOCK16;
		usb_data[2] = (address>>24)&0xff;
		usb_data[3] = (address>>16)&0xff;
		usb_data[4] = (address>>8)&0xff;
		usb_data[5] = (address)&0xff;
		bdm_usb_recv_ep0(usb_data);
		print( "TBLCF_READ_BLOCK16: Block read, size 0x%02X (0x%02X):\r\n",
		       blockSize, usb_data[0]);
		printDump(usb_data+1, blockSize);
      memcpy(buffer, usb_data+1, blockSize);
		byteCount -= blockSize;
		address   += blockSize;
      buffer    += blockSize;
		if (usb_data[0] != CMD_READ_MEMBLOCK16)
		   return(1);
	}
	if (byteCount) {  // Trailing odd byte
      print("TBLCF_READ_BLOCK16: Performing trailing 1 byte read\r\n");
      if (prefix(read_mem8)(address, buffer))
         return(1);
	}
	return(0);
}

/* reads the requested number of bytes from target memory from the supplied address and stores results into the user supplied buffer */
/* uses long word accesses */
/* if the address is not aligned byte/word reads are performed to align it */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(read_block32)( unsigned long int address,
                                            unsigned long int byteCount,
                                            unsigned char *buffer) {
   const unsigned maxBlockSize = ((MAX_DATA_SIZE-1)&0xfffC);
   unsigned int   blockSize;

   print( "TBLCF_READ_BLOCK32: Read 0x%08lX byte(s) from address 0x%08lX:\r\n",
	       byteCount, address);

	if (address&0x01) { // Odd address - read 1 byte to make even
		print("TBLCF_READ_BLOCK32: Address is odd, performing 1 byte read\r\n");
		if (prefix(read_mem8)(address, buffer))
		   return(1);
		byteCount--;
		address++;
		buffer++;
	}
	if (address&0x02) { // Not word aligned - read 1 word to align
		print("TBLCF_READ_BLOCK32: Address is not long aligned, performing 1 word read\r\n");
		if (prefix(read_mem16)(address, (unsigned int *)buffer))
		   return(1);
		byteCount -= 2;
		address   += 2;
		buffer    += 2;
	}
   while (byteCount >= 4) { // Read long words
      blockSize = byteCount;
      if (blockSize > maxBlockSize)
         blockSize = maxBlockSize;
      usb_data[0] = blockSize+1;  /* receive block of maximum size */
      usb_data[1] = CMD_READ_MEMBLOCK32;
      usb_data[2] = (address>>24)&0xff;
      usb_data[3] = (address>>16)&0xff;
      usb_data[4] = (address>>8)&0xff;
      usb_data[5] = (address)&0xff;
      bdm_usb_recv_ep0(usb_data);
      print( "TBLCF_READ_BLOCK32: Block read, size 0x%02X (0x%02X):\r\n",
             blockSize, usb_data[0]);
      printDump(usb_data+1, blockSize);
      memcpy(buffer, usb_data+1, blockSize);
      byteCount -= blockSize;
      address   += blockSize;
      buffer    += blockSize;
      if (usb_data[0] != CMD_READ_MEMBLOCK32)
         return(1);
   }
   if (byteCount > 0) { // Read trailing odd bytes
      usb_data[0] = byteCount+1;
      usb_data[1] = CMD_READ_MEMBLOCK8;
      usb_data[2] = (address>>24)&0xff;
      usb_data[3] = (address>>16)&0xff;
      usb_data[4] = (address>>8)&0xff;
      usb_data[5] = (address)&0xff;
      bdm_usb_recv_ep0(usb_data);
      print("TBLCF_READ_BLOCK32: Trailing block read, size 0x%02X (0x%02X):\r\n",
            byteCount, usb_data[0]);
      if (usb_data[0] != CMD_READ_MEMBLOCK8)
         return(1);
      memcpy(buffer, usb_data+1, byteCount);
   }
	return(0);
}

/* writes the requested number of bytes to target memory from the supplied address */
/* uses byte accesses only */
/* returns 0 on success and non-zero on failure
 * (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
USBDM_API unsigned char prefix(write_block8)( unsigned long int address,
                                            unsigned long int byteCount,
                                            unsigned char *buffer) {
   const unsigned maxBlockSize = ((MAX_DATA_SIZE-6)&0xfffe);
   unsigned int   blockSize;

   print("TBLCF_WRITE_BLOCK8: Write 0x%08lX byte(s) to address 0x%08lX:\r\n",
         byteCount, address);

   while (byteCount > 0) {
      blockSize = byteCount;
      if (blockSize > maxBlockSize)
         blockSize = maxBlockSize;
      usb_data[0] = blockSize+5;
      usb_data[1] = CMD_WRITE_MEMBLOCK8;
      usb_data[2] = (address>>24)&0xff;
      usb_data[3] = (address>>16)&0xff;
      usb_data[4] = (address>>8)&0xff;
      usb_data[5] = (address)&0xff;
      memcpy(usb_data+6, buffer, blockSize);
      print("TBLCF_WRITE_BLOCK8: Block write, size 0x%02X:\r\n", blockSize);
      printDump(usb_data+6, blockSize);
      bdm_usb_send_ep0(usb_data);
      if (usb_data[0] != CMD_WRITE_MEMBLOCK8)
         return(1);
#ifdef WRITE_BLOCK_CHECK
      if (prefix(get_last_sts)())
         return(1);
#endif
      byteCount -= blockSize;
      address   += blockSize;
      buffer    += blockSize;
   }
	return(0);
}

/* writes the requested number of bytes to target memory at the supplied address */
/* uses word accesses */
/* if the address is odd a byte write is performed to make it even before performing the word writes */
/* if there is an odd byte at the end of the block it is written using a byte write */
/* returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
USBDM_API unsigned char prefix(write_block16)( unsigned long int address,
                                             unsigned long int byteCount,
                                             unsigned char *buffer) {
   const unsigned maxBlockSize = ((MAX_DATA_SIZE-6)&0xfffe);
   unsigned int   blockSize;

	print("TBLCF_WRITE_BLOCK16: Write 0x%08lX byte(s) from address 0x%08lX:\r\n",
	      byteCount,address);

	if (address&0x01) {
		print("TBLCF_WRITE_BLOCK16: Address is odd, performing 1 byte write\r\n");
		prefix(write_mem8)(address, *buffer);
		#ifdef WRITE_BLOCK_CHECK
	   if (prefix(get_last_sts)())
			return(1);
		#endif
		byteCount--;
		address++;
		buffer++;
	}
   while (byteCount >= 2) { // Write words
      blockSize = byteCount;
      if (blockSize > maxBlockSize)
         blockSize = maxBlockSize;
      usb_data[0] = blockSize+5;
      usb_data[1] = CMD_WRITE_MEMBLOCK16;
      usb_data[2] = (address>>24)&0xff;
      usb_data[3] = (address>>16)&0xff;
      usb_data[4] = (address>>8)&0xff;
      usb_data[5] = (address)&0xff;
      memcpy(usb_data+6, buffer, blockSize);
      print("TBLCF_WRITE_BLOCK8: Block write, size 0x%02X:\r\n", blockSize);
      printDump(usb_data+6, blockSize);
      bdm_usb_send_ep0(usb_data);
      if (usb_data[0] != CMD_WRITE_MEMBLOCK16)
         return(1);
#ifdef WRITE_BLOCK_CHECK
      if (prefix(get_last_sts)())
         return(1);
#endif
      byteCount -= blockSize;
      address   += blockSize;
      buffer    += blockSize;
   }
	if (byteCount) {	/* leftover byte */
		print("TBLCF_WRITE_BLOCK16: Misaligned byte at the end of the block, performing 1 byte write\r\n");
		prefix(write_mem8)(address, *buffer);
		#ifdef WRITE_BLOCK_CHECK
			if (prefix(get_last_sts)())
			   return(1);
		#endif
	}
	return(0);
}

/* writes the requested number of bytes to target memory at the supplied address */
/* uses long word accesses */
/* if the address is not aligned byte/word writes are performed */
/* if there are odd bytes at the end of the block they is written using byte/word writes */
/* returns 0 on success and non-zero on failure (must be compiled with WRITE_BLOCK_CHECK, otherwise always returns 0) */
USBDM_API unsigned char prefix(write_block32)( unsigned long int address,
                                             unsigned long int byteCount,
                                             unsigned char *buffer) {
   const unsigned maxBlockSize = ((MAX_DATA_SIZE-6)&0xFFFC);
   unsigned int   blockSize;

	print("TBLCF_WRITE_BLOCK32: Write 0x%08lX byte(s) at address 0x%08lX:\r\n",byteCount,address);
	if (address&0x01) {
		print("TBLCF_WRITE_BLOCK32: Address is odd, performing 1 byte write\r\n");
		prefix(write_mem8)(address, *buffer);
		#ifdef WRITE_BLOCK_CHECK
      if (prefix(get_last_sts)())
         return(1);
		#endif
		byteCount--;
		address++;
		buffer++;
	}
	if (address&0x02) {
		print("TBLCF_WRITE_BLOCK32: Address is not aligned, performing 1 word write\r\n");
		prefix(write_mem16)(address, (*(buffer+0))*256+*(buffer+1));
		#ifdef WRITE_BLOCK_CHECK
      if (prefix(get_last_sts)())
         return(1);
		#endif
		byteCount-=2;
		address+=2;
		buffer+=2;
	}
   while (byteCount >= 4) { // Long word writes
      blockSize = byteCount;
      if (blockSize > maxBlockSize)
         blockSize = maxBlockSize;
      usb_data[0] = blockSize+5;
      usb_data[1] = CMD_WRITE_MEMBLOCK32;
      usb_data[2] = (address>>24)&0xff;
      usb_data[3] = (address>>16)&0xff;
      usb_data[4] = (address>>8)&0xff;
      usb_data[5] = (address)&0xff;
      memcpy(usb_data+6, buffer, blockSize);
      print("TBLCF_WRITE_BLOCK8: Block write, size 0x%02X:\r\n", blockSize);
      printDump(usb_data+6, blockSize);
      bdm_usb_send_ep0(usb_data);
      if (usb_data[0] != CMD_WRITE_MEMBLOCK32)
         return(1);
#ifdef WRITE_BLOCK_CHECK
      if (prefix(get_last_sts)())
         return(1);
#endif
      byteCount -= blockSize;
      address   += blockSize;
      buffer    += blockSize;
   }
	if (byteCount>=2) {						/* leftover word */
		print("TBLCF_WRITE_BLOCK32: Misaligned word at the end of the block, performing 1 word write\r\n");
		prefix(write_mem16)(address, (*(buffer+0))*256+*(buffer+1));
		#ifdef WRITE_BLOCK_CHECK
		if (prefix(get_last_sts)())
		   return(1);
		#endif
		address+=2;
		byteCount-=2;
		buffer+=2;
	}
	if (byteCount) {						/* leftover byte */
		print("TBLCF_WRITE_BLOCK32: Misaligned byte at the end of the block, performing 1 byte write\r\n");
		prefix(write_mem8)(address, *(buffer));
		#ifdef WRITE_BLOCK_CHECK
      if (prefix(get_last_sts)())
         return(1);
		#endif
	}
	return(0);
}

/* JTAG - go from RUN-TEST/IDLE to SHIFT-DR or SHIFT-IR state */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(jtag_sel_shift)(unsigned char mode) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_JTAG_GOTOSHIFT;
	usb_data[2]=mode;
	bdm_usb_recv_ep0(usb_data);
	print("TBLCF_JTAG_SEL_SHIFT: SHIFT-%cR\r\n",mode?'I':'D');
	return(!(usb_data[0]==CMD_JTAG_GOTOSHIFT));
}

/* JTAG - go from RUN-TEST/IDLE to TEST-LOGIC-RESET state */
/* returns 0 on success and non-zero on failure */
unsigned char USBDM_API prefix(jtag_sel_reset)(void) {
	usb_data[0]=1;	 /* get 1 byte */
	usb_data[1]=CMD_JTAG_GOTORESET;
	bdm_usb_recv_ep0(usb_data);
	print("TBLCF_JTAG_SEL_RESET\r\n");
	return(!(usb_data[0]==CMD_JTAG_GOTORESET));
}

/* JTAG - write data */
/* parameter exit: ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when done */
/* data: shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte */
USBDM_API void prefix(jtag_write)(unsigned char bit_count,
                                unsigned char exit,
                                unsigned char *buffer) {
	int i;
	usb_data[0]=(bit_count>>3)+((bit_count&0x07)!=0)+3;	 /* send all data bits & 3 more bytes */
	usb_data[1]=CMD_JTAG_WRITE;
	usb_data[2]=exit;
	usb_data[3]=bit_count;
	for (i=0;i<((bit_count>>3)+1);i++) usb_data[4+i]=*(buffer+i);	/* copy data */
	bdm_usb_send_ep0(usb_data);
	print("TBLCF_JTAG_WRITE: %d bits (%sexit to RUN-TEST/IDLE)\r\n",bit_count,exit?"":"NO ");
	printDump(usb_data+4,(bit_count>>3)+((bit_count&0x07)!=0));
}

/* JTAG - read data */
/* parameter exit: ==0 : stay in SHIFT-xx, !=0 : go to RUN-TEST/IDLE when done */
/* data: shifted in LSB (last byte) first, unused bits (if any) are in the MSB (first) byte */
/* returns 0 on success and non-zero on failure */
USBDM_API unsigned char prefix(jtag_read)( unsigned char bit_count,
                                         unsigned char exit,
                                         unsigned char *buffer) {
	int i;
	usb_data[0]=(bit_count>>3)+((bit_count&0x07)!=0)+1;	 /* get all data bits & the status byte */
	usb_data[1]=CMD_JTAG_READ;
	usb_data[2]=exit;
	usb_data[3]=bit_count;
	bdm_usb_recv_ep0(usb_data);
	for (i=0;i<((bit_count>>3)+((bit_count&0x07)!=0));i++) *(buffer+i)=usb_data[1+i];	/* copy data */
	print("TBLCF_JTAG_READ: %d bits (%sexit to RUN-TEST/IDLE)\r\n",bit_count,exit?"":"NO ");
	printDump(usb_data+1,(bit_count>>3)+((bit_count&0x07)!=0));
	return(!(usb_data[0]==CMD_JTAG_READ));
}

/*! Set BDM for ICP mode & immediately reboot
 *
 *  The BDM resets in ICP mode after this command
 *  @note Communication is lost.
 */
USBDM_API void prefix(setBoot)(void){

   usb_data[0] = 5;  // send 6 bytes
   usb_data[1] = CMD_SET_BOOT;
   usb_data[2] = 'B';
   usb_data[3] = 'O';
   usb_data[4] = 'O';
   usb_data[5] = 'T';

   (void)bdm_usb_send_ep0(usb_data); // USB transfer will be incomplete
}

//! ICP mode - reboot
//!
//! The BDM does a normal reset
//!
//! Usually used in ICP mode to reset to normal (BDM) mode.
//! @note Communication is lost.  \n
//!       This will do the usual flash validity checks before entering BDM mode.
//!
USBDM_API void prefix(icp_reboot)( void ) {

   bdm_usb_raw_send_ep0(ICP_REBOOT,
                        0,
                        0,
                        0,
                        NULL);
}


//! ICP mode - program BDM Flash memory
//!
//! @param addr    16-bit memory address
//! @param count   8-bit byte count
//! @param data    Pointer to buffer containing data
//!
USBDM_API unsigned int prefix(icp_program)(unsigned int  addr,
                                                  unsigned int  count,
                                                  unsigned char *data) {

   unsigned int  dataSize;     // Bytes to program this row
   unsigned char rc;
   unsigned int  temp;
            int  doBlock;

   // Have to program within a row in each iteration
   // so there are size and alignment issues.
   while (count > 0) {
      // Bytes to end of current row
      dataSize = ICP_JM60_ROWSIZE - (addr & (ICP_JM60_ROWSIZE-1));
      if (dataSize > count)  // Take down to number actually remaining
         dataSize = count;

      // Check if empty block (all 0xFF)
      doBlock = FALSE;
      for (temp = 0; temp<dataSize; temp++)
         doBlock = doBlock || (data[temp] != 0xFF);

      if (doBlock) { // Only program non-Blank blocks
         print("Programming block %4.4X-%4.4X\r\n", addr, addr+dataSize-1);
         rc = bdm_usb_raw_send_ep0(ICP_PROGRAM_ROW,
                                   addr&0xFFFF, // LSB address
                                   addr>>16,    // MSB address
                                   dataSize,
                                   data);
         if (rc != ICP_RC_OK)
            return rc;

         rc = prefix(icp_get_result)();

         if (rc != ICP_RC_OK)
            return rc;
      }
      else
         print("Skipping block    %4.4X-%4.4X\r\n", addr, addr+dataSize-1);

      data  += dataSize;   // update location in buffer
      addr  += dataSize;   // update address
      count -= dataSize;   // update count
   }
   return ICP_RC_OK;
}

//! ICP mode - verify BDM Flash memory
//!
//! @param addr    16-bit memory address
//! @param count   8-bit byte count
//! @param data    Pointer to buffer containing data
//!
USBDM_API unsigned int prefix(icp_verify)(unsigned int  addr,
                                         unsigned int  count,
                                         unsigned char *data) {
   unsigned       int  dataSize;
   unsigned       char rc;

   // Can verify MaxDataSize bytes at a time
   // Doesn't have alignment issues
   while (count > 0) {
      dataSize = count;
      if (dataSize > ICP_JM60_ROWSIZE)
         dataSize = ICP_JM60_ROWSIZE;

      print("Verifying block %4.4X-%4.4X\r\n", addr, addr+dataSize-1);
      rc = bdm_usb_raw_send_ep0(ICP_VERIFY_ROW,
                                addr&0xFFFF, // LSB address
                                addr>>16,    // MSB address
                                dataSize,
                                data);
      if (rc != ICP_RC_OK)
         return rc;

      rc = prefix(icp_get_result)();

      if (rc != ICP_RC_OK)
         return rc;

      data  += dataSize;   // update location in buffer
      addr  += dataSize;   // update address
      count -= dataSize;   // update count
   }
   return ICP_RC_OK;
}

//! ICP mode - erase BDM Flash memory
//!
//! @param addr    16-bit memory address
//! @param count   8-bit byte count
//!
USBDM_API unsigned int prefix(icp_erase)(unsigned int addr,
                                        unsigned int count) {
   unsigned int blockSize;
   unsigned char rc;

   // Flash memory is organised in blocks
   // Only an entire block can be erased at a time
   while (count > 0) {
      blockSize = count;
      if (blockSize > ICP_JM60_BLOCKSIZE)
         blockSize = ICP_JM60_BLOCKSIZE;

      print("Erasing block %4.4X-%4.4X\r\n", addr, addr+blockSize-1);
      rc = bdm_usb_raw_send_ep0(ICP_ERASE_PAGE,
                                addr&0xFFFF, // LSB address
                                addr>>16,    // MSB address
                                0,
                                NULL);
      if (rc != ICP_RC_OK)
         return rc;

      rc = prefix(icp_get_result)();

      if (rc != ICP_RC_OK)
         return rc;

      addr  += blockSize;   // update address
      count -= blockSize;   // update count
   }
   return ICP_RC_OK;
}

//! ICP mode - get result of last ICP command
//!
USBDM_API unsigned int prefix(icp_get_result)( void ) {

   unsigned char rc;
   unsigned char buff[10];

   rc = bdm_usb_raw_recv_ep0( ICP_GET_RESULT,
                              0,
                              0,
                              sizeof(buff),
                              buff);
   return buff[0];
}

#if defined (COMMANDLINE)
//! Execute debug command (various, see \ref DebugSubCommands)
//!
//! @return 16-bit value
//!
void bdm_debug(char *usb_data) {
   usb_data[0] = 7;  // receive up to 7 bytes
   usb_data[1] = CMD_DEBUG;
   bdm_usb_recv_ep0(usb_data);
}
#endif

//! Passes a set option command to the BDM
//!
//! @param option Option number
//! @param value  Value for the option
//!
void bdm_set_option(unsigned char option, unsigned int value) {
   usb_data[0] = 2;  // receive up to 2 bytes
   usb_data[1] = CMD_OPTION;
   usb_data[2] = option;
   usb_data[3] = (unsigned char) value&0xFF;
   usb_data[4] = (unsigned char) (value>>8)&0xFF;
   bdm_usb_recv_ep0(usb_data);
}

//! Initialise DLL
//!
#if !defined(COMMANDLINE)
static
#endif
void dll_initialize(HINSTANCE hDLLInst){

   openLogFile(NULL);

   bdm_usb_init();       /* initialise USB */

//   initialised = 1;
}

//! Un-initialise DLL
//!
#if !defined(COMMANDLINE)
static
#endif
void dll_uninitialize(void) {
   closeLogFile();
//   initialised = 0;
}

#if defined(DLL)

USBDM_API BOOL DllMain(HINSTANCE _hDLLInst,
                       DWORD fdwReason, LPVOID lpvReserved) {

   hDLLInst = _hDLLInst;

    switch (fdwReason) {
        case DLL_PROCESS_ATTACH:
         dll_initialize(hDLLInst);
            break;
        case DLL_PROCESS_DETACH:
         dll_uninitialize();
            break;
        case DLL_THREAD_ATTACH:
            break;
        case DLL_THREAD_DETACH:
            break;
    }
    return TRUE;
}

#endif
